@@ -69,7 +69,7 @@ pub enum AlterTableOperation {
     /// Add Partitions
     AddPartitions {
         if_not_exists: bool,
-        new_partitions: Vec<Expr>,
+        new_partitions: Vec<Partition>,
     },
     DropPartitions {
         partitions: Vec<Expr>,
@@ -119,8 +119,8 @@ impl fmt::Display for AlterTableOperation {
                 new_partitions,
             } => write!(
                 f,
-                "ADD{ine} PARTITION ({})",
-                display_comma_separated(new_partitions),
+                "ADD{ine} {}",
+                display_separated(new_partitions, " "),
                 ine = if *if_not_exists { " IF NOT EXISTS" } else { "" }
             ),
             AlterTableOperation::AddConstraint(c) => write!(f, "ADD {c}"),
@@ -517,6 +517,9 @@ pub struct ColumnDef {
 impl fmt::Display for ColumnDef {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{} {}", self.name, self.data_type)?;
+        if let Some(collation) = &self.collation {
+            write!(f, " COLLATE {collation}")?;
+        }
         for option in &self.options {
             write!(f, " {option}")?;
         }
@@ -596,6 +599,9 @@ pub enum ColumnOption {
         generated_as: GeneratedAs,
         sequence_options: Option<Vec<SequenceOptions>>,
         generation_expr: Option<Expr>,
+        generation_expr_mode: Option<GeneratedExpressionMode>,
+        /// false if 'GENERATED ALWAYS' is skipped (option starts with AS)
+        generated_keyword: bool,
     },
 }
 
@@ -636,9 +642,30 @@ impl fmt::Display for ColumnOption {
                 generated_as,
                 sequence_options,
                 generation_expr,
-            } => match generated_as {
-                GeneratedAs::Always => {
-                    write!(f, "GENERATED ALWAYS AS IDENTITY")?;
+                generation_expr_mode,
+                generated_keyword,
+            } => {
+                if let Some(expr) = generation_expr {
+                    let modifier = match generation_expr_mode {
+                        None => "",
+                        Some(GeneratedExpressionMode::Virtual) => " VIRTUAL",
+                        Some(GeneratedExpressionMode::Stored) => " STORED",
+                    };
+                    if *generated_keyword {
+                        write!(f, "GENERATED ALWAYS AS ({expr}){modifier}")?;
+                    } else {
+                        write!(f, "AS ({expr}){modifier}")?;
+                    }
+                    Ok(())
+                } else {
+                    // Like Postgres - generated from sequence
+                    let when = match generated_as {
+                        GeneratedAs::Always => "ALWAYS",
+                        GeneratedAs::ByDefault => "BY DEFAULT",
+                        // ExpStored goes with an expression, handled above
+                        GeneratedAs::ExpStored => unreachable!(),
+                    };
+                    write!(f, "GENERATED {when} AS IDENTITY")?;
                     if sequence_options.is_some() {
                         let so = sequence_options.as_ref().unwrap();
                         if !so.is_empty() {
@@ -653,33 +680,13 @@ impl fmt::Display for ColumnOption {
                     }
                     Ok(())
                 }
-                GeneratedAs::ByDefault => {
-                    write!(f, "GENERATED BY DEFAULT AS IDENTITY")?;
-                    if sequence_options.is_some() {
-                        let so = sequence_options.as_ref().unwrap();
-                        if !so.is_empty() {
-                            write!(f, " (")?;
-                        }
-                        for sequence_option in so {
-                            write!(f, "{sequence_option}")?;
-                        }
-                        if !so.is_empty() {
-                            write!(f, " )")?;
-                        }
-                    }
-                    Ok(())
-                }
-                GeneratedAs::ExpStored => {
-                    let expr = generation_expr.as_ref().unwrap();
-                    write!(f, "GENERATED ALWAYS AS ({expr}) STORED")
-                }
-            },
+            }
         }
     }
 }
 
 /// `GeneratedAs`s are modifiers that follow a column option in a `generated`.
-/// 'ExpStored' is PostgreSQL specific
+/// 'ExpStored' is used for a column generated from an expression and stored.
 #[derive(Debug, Clone, PartialEq, PartialOrd, Eq, Ord, Hash)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "visitor", derive(Visit, VisitMut))]
@@ -689,6 +696,16 @@ pub enum GeneratedAs {
     ExpStored,
 }
 
+/// `GeneratedExpressionMode`s are modifiers that follow an expression in a `generated`.
+/// No modifier is typically the same as Virtual.
+#[derive(Debug, Clone, PartialEq, PartialOrd, Eq, Ord, Hash)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "visitor", derive(Visit, VisitMut))]
+pub enum GeneratedExpressionMode {
+    Virtual,
+    Stored,
+}
+
 fn display_constraint_name(name: &'_ Option<Ident>) -> impl fmt::Display + '_ {
     struct ConstraintName<'a>(&'a Option<Ident>);
     impl<'a> fmt::Display for ConstraintName<'a> {
@@ -768,3 +785,21 @@ impl fmt::Display for UserDefinedTypeCompositeAttributeDef {
         Ok(())
     }
 }
+
+/// PARTITION statement used in ALTER TABLE et al. such as in Hive SQL
+#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "visitor", derive(Visit, VisitMut))]
+pub struct Partition {
+    pub partitions: Vec<Expr>,
+}
+
+impl fmt::Display for Partition {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            f,
+            "PARTITION ({})",
+            display_comma_separated(&self.partitions)
+        )
+    }
+}
