@@ -64,6 +64,9 @@ macro_rules! dialect_of {
 /// custom extensions or various historical reasons. This trait
 /// encapsulates the parsing differences between dialects.
 ///
+/// [`GenericDialect`] is the most permissive dialect, and parses the union of
+/// all the other dialects, when there is no ambiguity.
+///
 /// # Examples
 /// Most users create a [`Dialect`] directly, as shown on the [module
 /// level documentation]:
@@ -86,13 +89,22 @@ macro_rules! dialect_of {
 ///
 /// [module level documentation]: crate
 pub trait Dialect: Debug + Any {
+    /// Determine the [`TypeId`] of this dialect.
+    ///
+    /// By default, return the same [`TypeId`] as [`Any::type_id`]. Can be overriden
+    /// by dialects that behave like other dialects
+    /// (for example when wrapping a dialect).
+    fn dialect(&self) -> TypeId {
+        self.type_id()
+    }
+
     /// Determine if a character starts a quoted identifier. The default
     /// implementation, accepting "double quoted" ids is both ANSI-compliant
     /// and appropriate for most dialects (with the notable exception of
     /// MySQL, MS SQL, and sqlite). You can accept one of characters listed
     /// in `Word::matching_end_quote` here
     fn is_delimited_identifier_start(&self, ch: char) -> bool {
-        ch == '"'
+        ch == '"' || ch == '`'
     }
     /// Determine if quoted characters are proper for identifier
     fn is_proper_identifier_inside_quotes(&self, mut _chars: Peekable<Chars<'_>>) -> bool {
@@ -121,6 +133,19 @@ pub trait Dialect: Debug + Any {
     fn supports_substring_from_for_expr(&self) -> bool {
         true
     }
+    /// Returns true if the dialect supports `(NOT) IN ()` expressions
+    fn supports_in_empty_list(&self) -> bool {
+        false
+    }
+    /// Returns true if the dialect supports `BEGIN {DEFERRED | IMMEDIATE | EXCLUSIVE} [TRANSACTION]` statements
+    fn supports_start_transaction_modifier(&self) -> bool {
+        false
+    }
+    /// Returns true if the dialect has a CONVERT function which accepts a type first
+    /// and an expression second, e.g. `CONVERT(varchar, 1)`
+    fn convert_type_before_value(&self) -> bool {
+        false
+    }
     /// Dialect-specific prefix parser override
     fn parse_prefix(&self, _parser: &mut Parser) -> Option<Result<Expr, ParserError>> {
         // return None to fall back to the default behavior
@@ -152,7 +177,7 @@ impl dyn Dialect {
     #[inline]
     pub fn is<T: Dialect>(&self) -> bool {
         // borrowed from `Any` implementation
-        TypeId::of::<T>() == self.type_id()
+        TypeId::of::<T>() == self.dialect()
     }
 }
 
@@ -236,4 +261,98 @@ mod tests {
     fn parse_dialect(v: &str) -> Box<dyn Dialect> {
         dialect_from_str(v).unwrap()
     }
+
+    #[test]
+    fn parse_with_wrapped_dialect() {
+        /// Wrapper for a dialect. In a real-world example, this wrapper
+        /// would tweak the behavior of the dialect. For the test case,
+        /// it wraps all methods unaltered.
+        #[derive(Debug)]
+        struct WrappedDialect(MySqlDialect);
+
+        impl Dialect for WrappedDialect {
+            fn dialect(&self) -> std::any::TypeId {
+                self.0.dialect()
+            }
+
+            fn is_identifier_start(&self, ch: char) -> bool {
+                self.0.is_identifier_start(ch)
+            }
+
+            fn is_delimited_identifier_start(&self, ch: char) -> bool {
+                self.0.is_delimited_identifier_start(ch)
+            }
+
+            fn is_proper_identifier_inside_quotes(
+                &self,
+                chars: std::iter::Peekable<std::str::Chars<'_>>,
+            ) -> bool {
+                self.0.is_proper_identifier_inside_quotes(chars)
+            }
+
+            fn supports_filter_during_aggregation(&self) -> bool {
+                self.0.supports_filter_during_aggregation()
+            }
+
+            fn supports_within_after_array_aggregation(&self) -> bool {
+                self.0.supports_within_after_array_aggregation()
+            }
+
+            fn supports_group_by_expr(&self) -> bool {
+                self.0.supports_group_by_expr()
+            }
+
+            fn supports_substring_from_for_expr(&self) -> bool {
+                self.0.supports_substring_from_for_expr()
+            }
+
+            fn supports_in_empty_list(&self) -> bool {
+                self.0.supports_in_empty_list()
+            }
+
+            fn convert_type_before_value(&self) -> bool {
+                self.0.convert_type_before_value()
+            }
+
+            fn parse_prefix(
+                &self,
+                parser: &mut sqlparser::parser::Parser,
+            ) -> Option<Result<Expr, sqlparser::parser::ParserError>> {
+                self.0.parse_prefix(parser)
+            }
+
+            fn parse_infix(
+                &self,
+                parser: &mut sqlparser::parser::Parser,
+                expr: &Expr,
+                precedence: u8,
+            ) -> Option<Result<Expr, sqlparser::parser::ParserError>> {
+                self.0.parse_infix(parser, expr, precedence)
+            }
+
+            fn get_next_precedence(
+                &self,
+                parser: &sqlparser::parser::Parser,
+            ) -> Option<Result<u8, sqlparser::parser::ParserError>> {
+                self.0.get_next_precedence(parser)
+            }
+
+            fn parse_statement(
+                &self,
+                parser: &mut sqlparser::parser::Parser,
+            ) -> Option<Result<Statement, sqlparser::parser::ParserError>> {
+                self.0.parse_statement(parser)
+            }
+
+            fn is_identifier_part(&self, ch: char) -> bool {
+                self.0.is_identifier_part(ch)
+            }
+        }
+
+        let statement = r#"SELECT 'Wayne\'s World'"#;
+        let res1 = Parser::parse_sql(&MySqlDialect {}, statement);
+        let res2 = Parser::parse_sql(&WrappedDialect(MySqlDialect {}), statement);
+        assert!(res1.is_ok());
+        assert_eq!(res1, res2);
+    }
 }
