@@ -7,7 +7,7 @@ use serde::{Deserialize, Serialize};
 use prqlc_ast::expr::generic;
 use prqlc_ast::{Ident, Literal, Span, Ty};
 
-pub use prqlc_ast::expr::{BinOp, UnOp};
+use crate::codegen::write_ty;
 
 use super::{Lineage, TransformCall};
 
@@ -15,7 +15,7 @@ use super::{Lineage, TransformCall};
 
 /// Expr is anything that has a value and thus a type.
 /// If it cannot contain nested Exprs, is should be under [ExprKind::Literal].
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Clone, PartialEq, Serialize, Deserialize)]
 pub struct Expr {
     #[serde(flatten)]
     pub kind: ExprKind,
@@ -34,10 +34,6 @@ pub struct Expr {
     #[serde(skip_serializing_if = "Option::is_none")]
     pub target_id: Option<usize>,
 
-    /// For [ExprKind::All], these are ids of included nodes
-    #[serde(skip_serializing_if = "Vec::is_empty", default)]
-    pub target_ids: Vec<usize>,
-
     /// Type of expression this node represents.
     /// [None] means that type should be inferred.
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -64,8 +60,8 @@ pub struct Expr {
 pub enum ExprKind {
     Ident(Ident),
     All {
-        within: Ident,
-        except: Vec<Expr>,
+        within: Box<Expr>,
+        except: Box<Expr>,
     },
     Literal(Literal),
 
@@ -157,3 +153,44 @@ impl From<Func> for ExprKind {
         ExprKind::Func(Box::new(value))
     }
 }
+
+impl std::fmt::Debug for Expr {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut ds = f.debug_struct("Expr");
+
+        if let Some(x) = &self.span {
+            ds.field("span", x);
+        }
+        ds.field("kind", &self.kind);
+        if let Some(x) = &self.alias {
+            ds.field("alias", x);
+        }
+        if let Some(x) = &self.id {
+            ds.field("id", x);
+        }
+        if let Some(x) = &self.target_id {
+            ds.field("target_id", x);
+        }
+        if self.needs_window {
+            ds.field("needs_window", &self.needs_window);
+        }
+        if self.flatten {
+            ds.field("flatten", &self.flatten);
+        }
+        if let Some(x) = &self.ty {
+            // DebugTy is needed to get around string quotes that
+            // would be printed if we Debug-ed the string directly.
+            struct DebugTy<'a>(&'a Ty);
+            impl std::fmt::Debug for DebugTy<'_> {
+                fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+                    f.write_str(&write_ty(self.0))
+                }
+            }
+            ds.field("ty", &DebugTy(x));
+        }
+        if let Some(x) = &self.lineage {
+            ds.field("lineage", x);
+        }
+        ds.finish()
+    }
+}
