@@ -1,30 +1,24 @@
 use enum_as_inner::EnumAsInner;
 use serde::{Deserialize, Serialize};
 
-use prql_ast::Span;
+use prqlc_ast::{Span, Ty};
 
-pub use prql_ast::stmt::QueryDef;
+pub use prqlc_ast::stmt::QueryDef;
 
 use super::expr::Expr;
 
-#[derive(Debug, PartialEq, Eq, Clone, Serialize, Deserialize)]
-pub enum VarDefKind {
-    Let,
-    Into,
-    Main,
-}
-
-// The following code is tested by the tests_misc crate to match stmt.rs in prql_ast.
+// The following code is tested by the tests_misc crate to match stmt.rs in prqlc_ast.
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 pub struct Stmt {
-    #[serde(skip)]
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub id: Option<usize>,
     #[serde(flatten)]
     pub kind: StmtKind,
-    #[serde(skip)]
+    #[serde(skip_serializing_if = "Option::is_none")]
     pub span: Option<Span>,
 
+    #[serde(skip_serializing_if = "Vec::is_empty", default)]
     pub annotations: Vec<Annotation>,
 }
 
@@ -38,16 +32,17 @@ pub enum StmtKind {
 
 #[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct VarDef {
-    pub name: Option<String>,
+    pub name: String,
     pub value: Box<Expr>,
-    pub ty_expr: Option<Box<Expr>>,
-    pub kind: VarDefKind,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub ty: Option<Ty>,
 }
 
 #[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
 pub struct TypeDef {
     pub name: String,
-    pub value: Option<Box<Expr>>,
+    pub value: Option<Ty>,
 }
 
 #[derive(Debug, PartialEq, Clone, Serialize, Deserialize)]
