@@ -16,7 +16,9 @@ mod tokenizer;
 /// Formats whitespace in a SQL string to make it easier to read.
 /// Optionally replaces parameter placeholders with `params`.
 pub fn format(query: &str, params: &QueryParams, options: FormatOptions) -> String {
-    let tokens = tokenizer::tokenize(query);
+    let named_placeholders = matches!(params, QueryParams::Named(_));
+
+    let tokens = tokenizer::tokenize(query, named_placeholders);
     formatter::format(&tokens, params, options)
 }
 
@@ -1067,13 +1069,29 @@ mod tests {
         assert_eq!(format(input, &QueryParams::None, options), expected);
     }
 
+    #[test]
+    fn it_recognizes_dollar_sign_alphanumeric_placeholders() {
+        let input = "SELECT $hash, $foo, $bar;";
+        let options = FormatOptions::default();
+        let expected = indoc!(
+            "
+            SELECT
+              $hash,
+              $foo,
+              $bar;"
+        );
+
+        assert_eq!(format(input, &QueryParams::None, options), expected);
+    }
+
     #[test]
     fn it_recognizes_dollar_sign_numbered_placeholders_with_param_values() {
-        let input = "SELECT $2, $3, $1;";
+        let input = "SELECT $2, $3, $1, $named, $4, $alias;";
         let params = vec![
             "first".to_string(),
             "second".to_string(),
             "third".to_string(),
+            "4th".to_string(),
         ];
         let options = FormatOptions::default();
         let expected = indoc!(
@@ -1081,7 +1099,10 @@ mod tests {
             SELECT
               second,
               third,
-              first;"
+              first,
+              $named,
+              4th,
+              $alias;"
         );
 
         assert_eq!(
@@ -1090,6 +1111,31 @@ mod tests {
         );
     }
 
+    #[test]
+    fn it_recognizes_dollar_sign_alphanumeric_placeholders_with_param_values() {
+        let input = "SELECT $hash, $salt, $1, $2;";
+        let params = vec![
+            ("hash".to_string(), "hash value".to_string()),
+            ("salt".to_string(), "salt value".to_string()),
+            ("1".to_string(), "number 1".to_string()),
+            ("2".to_string(), "number 2".to_string()),
+        ];
+        let options = FormatOptions::default();
+        let expected = indoc!(
+            "
+            SELECT
+              hash value,
+              salt value,
+              number 1,
+              number 2;"
+        );
+
+        assert_eq!(
+            format(input, &QueryParams::Named(params), options),
+            expected
+        );
+    }
+
     #[test]
     fn it_formats_query_with_go_batch_separator() {
         let input = "SELECT 1 GO SELECT 2";
@@ -1401,13 +1447,14 @@ mod tests {
 
     #[test]
     fn it_recognizes_scientific_notation() {
-        let input = "SELECT *, 1e-7 as small, 1e+7 as large FROM t";
+        let input = "SELECT *, 1e-7 as small, 1e2 as medium, 1e+7 as large FROM t";
         let options = FormatOptions::default();
         let expected = indoc!(
             "
             SELECT
               *,
               1e-7 as small,
+              1e2 as medium,
               1e+7 as large
             FROM
               t"
@@ -1415,4 +1462,46 @@ mod tests {
 
         assert_eq!(format(input, &QueryParams::None, options), expected);
     }
+
+    #[test]
+    fn it_keeps_double_dollar_signs_together() {
+        let input = "CREATE FUNCTION abc() AS $$ SELECT * FROM table $$ LANGUAGE plpgsql;";
+        let options = FormatOptions::default();
+        let expected = indoc!(
+            "
+            CREATE FUNCTION abc() AS
+            $$
+            SELECT
+              *
+            FROM
+              table
+            $$
+            LANGUAGE plpgsql;"
+        );
+
+        assert_eq!(format(input, &QueryParams::None, options), expected);
+    }
+
+    #[test]
+    fn it_formats_pgplsql() {
+        let input = "CREATE FUNCTION abc() AS $$ DECLARE a int := 1; b int := 2; BEGIN SELECT * FROM table $$ LANGUAGE plpgsql;";
+        let options = FormatOptions::default();
+        let expected = indoc!(
+            "
+            CREATE FUNCTION abc() AS
+            $$
+            DECLARE
+            a int := 1;
+            b int := 2;
+            BEGIN
+            SELECT
+              *
+            FROM
+              table
+            $$
+            LANGUAGE plpgsql;"
+        );
+
+        assert_eq!(format(input, &QueryParams::None, options), expected);
+    }
 }
