@@ -4,8 +4,8 @@ use anyhow::{anyhow, bail, Result};
 use itertools::Itertools;
 use regex::Regex;
 use sqlparser::ast::{
-    self as sql_ast, BinaryOperator, DateTimeField, Function, FunctionArg, FunctionArgExpr,
-    ObjectName, OrderByExpr, SelectItem, Top, UnaryOperator, Value, WindowFrameBound, WindowSpec,
+    self as sql_ast, BinaryOperator, DateTimeField, Fetch, Function, FunctionArg, FunctionArgExpr,
+    ObjectName, OrderByExpr, SelectItem, UnaryOperator, Value, WindowFrameBound, WindowSpec,
 };
 use std::cmp::Ordering;
 
@@ -14,7 +14,7 @@ use crate::ir::pl::{self, Ident, Literal};
 use crate::ir::rq::*;
 use crate::sql::srq::context::ColumnDecl;
 use crate::utils::{OrMap, VALID_IDENT};
-use crate::{Error, Span, WithErrorInfo};
+use crate::{Error, Reason, Span, WithErrorInfo};
 use prqlc_ast::expr::generic::{InterpolateItem, Range};
 
 use super::gen_projection::try_into_exprs;
@@ -103,6 +103,10 @@ pub(super) fn translate_expr(expr: Expr, ctx: &mut Context) -> Result<ExprOrSour
                     }
                 }
                 "std.concat" => return Ok(process_concat(&expr, ctx)?.into()),
+                "std.array_in" => return Ok(process_array_in(args, ctx)?.into()),
+                "std.date.to_text" => {
+                    return Ok(process_date_to_text(&expr, name, args, ctx)?.into())
+                }
                 _ => match try_into_between(expr.clone(), ctx)? {
                     Some(between_expr) => return Ok(between_expr.into()),
                     None => {
@@ -116,6 +120,13 @@ pub(super) fn translate_expr(expr: Expr, ctx: &mut Context) -> Result<ExprOrSour
             }
             super::operators::translate_operator_expr(expr, ctx)?
         }
+        ExprKind::Array(_) => {
+            return Err(Error::new(Reason::Unexpected {
+                found: "array of values (not supported here)".to_string(),
+            })
+            .with_span(expr.span)
+            .into());
+        }
     })
 }
 
@@ -146,6 +157,69 @@ fn process_null(name: &str, args: &[Expr], ctx: &mut Context) -> Result<sql_ast:
     }
 }
 
+/// Translates into IN (v1, v2, ...) if possible
+fn process_array_in(args: &[Expr], ctx: &mut Context) -> Result<sql_ast::Expr> {
+    match args {
+        [col_expr @ Expr {
+            kind: ExprKind::ColumnRef(_),
+            ..
+        }, Expr {
+            kind: ExprKind::Array(in_values),
+            ..
+        }] => Ok(sql_ast::Expr::InList {
+            expr: Box::new(translate_expr(col_expr.clone(), ctx)?.into_ast()),
+            list: in_values
+                .iter()
+                .map(|a| Ok(translate_expr(a.clone(), ctx)?.into_ast()))
+                .collect::<Result<Vec<sql_ast::Expr>>>()?,
+            negated: false,
+        }),
+        _ => panic!("args to `std.array_in` must be a column ref and an array"),
+    }
+}
+
+/// Translates PRQL date format (based on `chrono` crate) to dialect specific date format
+/// For now only date format as string literal is supported
+fn process_date_to_text(
+    expr: &Expr,
+    op_name: &str,
+    args: &[Expr],
+    ctx: &mut Context,
+) -> Result<sql_ast::Expr> {
+    if let [date_format_exp @ Expr {
+        kind: ExprKind::Literal(Literal::String(date_format)),
+        ..
+    }, col_expr] = args
+    {
+        let expr = Expr {
+            kind: ExprKind::Operator {
+                name: op_name.to_string(),
+                args: vec![
+                    Expr {
+                        kind: ExprKind::Literal(Literal::String(
+                            ctx.dialect
+                                .translate_prql_date_format(date_format)
+                                .map_err(|e| {
+                                    Error::new_simple(e).with_span(date_format_exp.span)
+                                })?,
+                        )),
+                        span: date_format_exp.span,
+                    },
+                    col_expr.clone(),
+                ],
+            },
+            ..expr.clone()
+        };
+        Ok(super::operators::translate_operator_expr(expr, ctx)?.into_ast())
+    } else {
+        Err(
+            Error::new_simple("`std.date.to_text` only supports a string literal as format")
+                .with_span(expr.span)
+                .into(),
+        )
+    }
+}
+
 fn process_concat(expr: &Expr, ctx: &mut Context) -> Result<sql_ast::Expr> {
     if ctx.dialect.has_concat_function() {
         let concat_args = collect_concat_args(expr);
@@ -165,6 +239,8 @@ fn process_concat(expr: &Expr, ctx: &mut Context) -> Result<sql_ast::Expr> {
             distinct: false,
             special: false,
             order_by: vec![],
+            filter: None,
+            null_treatment: None,
         }))
     } else {
         let concat_args = collect_concat_args(expr);
@@ -382,6 +458,8 @@ fn translate_datetime_literal_with_sqlite_function(
         distinct: false,
         special: false,
         order_by: vec![],
+        filter: None,
+        null_treatment: None,
     })
 }
 
@@ -527,10 +605,10 @@ pub(super) fn expr_of_i64(number: i64) -> sql_ast::Expr {
     ))
 }
 
-pub(super) fn top_of_i64(take: i64, ctx: &mut Context) -> Top {
+pub(super) fn fetch_of_i64(take: i64, ctx: &mut Context) -> Fetch {
     let kind = ExprKind::Literal(Literal::Integer(take));
     let expr = Expr { kind, span: None };
-    Top {
+    Fetch {
         quantity: Some(translate_expr(expr, ctx).unwrap().into_ast()),
         with_ties: false,
         percent: false,
@@ -724,20 +802,19 @@ pub(super) fn translate_operand(
 /// For an operation represented as `a child b` with a surrounding parent
 /// operation (e.g., `(a child b) parent c` or `a parent (b child c)`):
 ///
-/// 1. When the child operator has higher precedence than the parent:
-///    - Parentheses are not required.
+/// 1. When the child operator has higher precedence than the parent,
+///    parentheses *are not* required.
 ///
-/// 2. When the child operator has lower precedence than the parent:
-///    - Parentheses are required.
+/// 2. When the child operator has lower precedence than the parent,
+///    parentheses *are* required.
 ///
-/// 3. When the child and parent operators have the same precedence, parentheses
-///    are not required if:
-///
-///    a. The parent is "Both" associative — e.g. `a + (b - c)`, but not `a - (b
-///    - c)`,
-///
-///    b. or the child is on the (left,right) and the child is (left,right)
-///    associative — e.g. `(a - b) - c` — but not `a - (b - c)`
+/// 3. When the child and parent operators have the same precedence, the child
+///    is on the {left,right} and the parent is {left,right} associative,
+///    parentheses are not required. Some examples of when parentheses are not required:
+///    - `(a - b) - c` & `(a + b) - c` — as opposed to `a - (b - c)`
+///    - `a + (b - c)` & `a + (b + c)` — as opposed to `a - (b + c)` & `a - (b - c)`
+///         
+///       
 //
 // If it were possible to evaluate this with less context that would be
 // preferable, but it's not clear how to do that. (For example, even if we
@@ -755,8 +832,8 @@ fn needs_parentheses(
     parent_associativity: Associativity,
 ) -> bool {
     let rule_3a = matches!(parent_associativity, Associativity::Both);
-    let rule_3b_left = is_left && expr.left_associative();
-    let rule_3b_right = !is_left && expr.right_associative();
+    let rule_3b_left = is_left && parent_associativity.left_associative();
+    let rule_3b_right = !is_left && parent_associativity.right_associative();
 
     match expr.binding_strength().cmp(&parent_strength) {
         // Rule 1
@@ -772,6 +849,7 @@ fn needs_parentheses(
 /// Note that there's no exponent symbol in SQL, so we don't seem to require a `Right` variant.
 /// https://en.wikipedia.org/wiki/Operator_associativity
 #[allow(dead_code)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum Associativity {
     Left,
     /// `Both` means mathematically associative, like `+` or `*`
@@ -779,30 +857,37 @@ pub enum Associativity {
     Right,
 }
 
+impl Associativity {
+    /// Returns true iff `a + b + c = (a + b) + c`
+    fn left_associative(&self) -> bool {
+        matches!(self, Associativity::Left | Associativity::Both)
+    }
+
+    /// Returns true iff `a + b + c = a + (b + c)`
+    fn right_associative(&self) -> bool {
+        matches!(self, Associativity::Right | Associativity::Both)
+    }
+}
+
 trait SQLExpression {
-    /// Returns binding strength of an SQL expression
+    /// Returns binding strength of a SQL expression
     /// https://www.postgresql.org/docs/14/sql-syntax-lexical.html#id-1.5.3.5.13.2
     /// https://docs.microsoft.com/en-us/sql/t-sql/language-elements/operator-precedence-transact-sql?view=sql-server-ver16
     fn binding_strength(&self) -> i32;
 
+    /// Default to `Both`, but expected to be overwritten by concrete types
     fn associativity(&self) -> Associativity {
         Associativity::Both
     }
 
     /// Returns true iff `a + b + c = (a + b) + c`
     fn left_associative(&self) -> bool {
-        matches!(
-            self.associativity(),
-            Associativity::Left | Associativity::Both
-        )
+        self.associativity().left_associative()
     }
 
     /// Returns true iff `a + b + c = a + (b + c)`
     fn right_associative(&self) -> bool {
-        matches!(
-            self.associativity(),
-            Associativity::Right | Associativity::Both
-        )
+        self.associativity().right_associative()
     }
 }
 
@@ -850,7 +935,7 @@ impl SQLExpression for BinaryOperator {
     fn associativity(&self) -> Associativity {
         use BinaryOperator::*;
         match self {
-            Minus | Divide => Associativity::Left,
+            Minus | Divide | Modulo => Associativity::Left,
             _ => Associativity::Both,
         }
     }
@@ -953,70 +1038,70 @@ mod test {
             Range { start, end }
         }
 
-        let range1 = from_ints(Some(1), Some(10));
-        let range2 = from_ints(Some(5), Some(6));
-        let range3 = from_ints(Some(5), None);
-        let range4 = from_ints(None, Some(8));
-        let range5 = from_ints(Some(5), Some(5));
+        let range_1_10 = from_ints(Some(1), Some(10));
+        let range_5_6 = from_ints(Some(5), Some(6));
+        let range_5_inf = from_ints(Some(5), None);
+        let range_inf_8 = from_ints(None, Some(8));
+        let range_5_5 = from_ints(Some(5), Some(5));
 
-        assert!(range_of_ranges(vec![range1.clone()])?.end.is_some());
+        assert!(range_of_ranges(vec![range_1_10.clone()])?.end.is_some());
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range1.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_1_10.clone()])?, @r###"
         ---
         start: 1
         end: 10
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range1.clone(), range1.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_1_10.clone(), range_1_10.clone()])?, @r###"
         ---
         start: 1
         end: 10
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range1.clone(), range2.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_1_10.clone(), range_5_6.clone()])?, @r###"
         ---
         start: 5
         end: 6
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range2.clone(), range1.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_5_6.clone(), range_1_10.clone()])?, @r###"
         ---
         start: 5
         end: 6
         "###);
 
         // empty range
-        assert_yaml_snapshot!(range_of_ranges(vec![range2.clone(), range2.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_5_6.clone(), range_5_6.clone()])?, @r###"
         ---
         start: ~
         end: 0
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range3.clone(), range3.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_5_inf.clone(), range_5_inf.clone()])?, @r###"
         ---
         start: 9
         end: ~
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range1, range3])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_1_10, range_5_inf])?, @r###"
         ---
         start: 5
         end: 10
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range2, range4.clone()])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_5_6, range_inf_8.clone()])?, @r###"
         ---
         start: 5
         end: 6
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range4.clone(), range4])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_inf_8.clone(), range_inf_8])?, @r###"
         ---
         start: ~
         end: 8
         "###);
 
-        assert_yaml_snapshot!(range_of_ranges(vec![range5])?, @r###"
+        assert_yaml_snapshot!(range_of_ranges(vec![range_5_5])?, @r###"
         ---
         start: 5
         end: 5
@@ -1024,147 +1109,4 @@ mod test {
 
         Ok(())
     }
-
-    #[test]
-    fn test_translate_datetime_literal_with_sqlite_function() -> Result<()> {
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Date,
-                "2020-01-01".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: DATE
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: 2020-01-01
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Time(None, sql_ast::TimezoneInfo::None),
-                "03:05".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: TIME
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: "03:05"
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Time(None, sql_ast::TimezoneInfo::None),
-                "03:05+08:00".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: TIME
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: "03:05+08:00"
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Time(None, sql_ast::TimezoneInfo::None),
-                "03:05+0800".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: TIME
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: "03:05+08:00"
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Timestamp(None, sql_ast::TimezoneInfo::None),
-                "2021-03-14T03:05+0800".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: DATETIME
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: "2021-03-14T03:05+08:00"
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        assert_yaml_snapshot!(
-                translate_datetime_literal_with_sqlite_function(
-                sql_ast::DataType::Timestamp(None, sql_ast::TimezoneInfo::None),
-                "2021-03-14T03:05+08:00".to_string(),
-            ),
-            @r###"
-        ---
-        Function:
-          name:
-            - value: DATETIME
-              quote_style: ~
-          args:
-            - Unnamed:
-                Expr:
-                  Value:
-                    SingleQuotedString: "2021-03-14T03:05+08:00"
-          over: ~
-          distinct: false
-          special: false
-          order_by: []
-        "###
-        );
-
-        Ok(())
-    }
 }
